/*
 * application.h
 * Copyright 2005 Red Hat Inc., Durham, North Carolina.
 * All Rights Reserved.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * Authors:
 *     Steve Grubb <sgrubb@redhat.com>
 */
#ifndef APPLICATION_H
#define APPLICATION_H
#include <string>
#include "circularqueue.h"
#include "component.h"
#include "event.h"

/**
 * Class Application
 * The application class follows the Singleton Pattern. It is accessible
 * from any class. Therefore, care should be taken to make sure that proper
 * locking is used inside it.  The class also serves as the owner of the 
 * main event queue. This is used to serialize all events handled by the
 * dispatcher.
 */
class Application : public Component {
/**
 * Public stuff
 */
public:
	/**
	 * Fields
	 */
	/**
	 * 
	 */
	/**
	 * Constructors
	 */
	/**
	 * Accessor Methods
	 */
	/**
	 * Operations
	 */
	/**
	 * This method is part of the Singleton Pattern. It is used to
	 * create the class or return a refernce to it.
	 */
	static Application *instance ()
	{
		if (!_instance) {
			static Application theInstance;
			_instance = &theInstance;
		}
		return _instance;
	}

	/**
	 * virtual functions inherited from Component
	 */	
	virtual int init(const char *config);
	virtual void destroy();
	virtual void suspend();
	virtual void resume();
	virtual const char *get_config();
	
	/**
	 * This method is used to tell the application class that a
	 * component wants to receive events.
	 */
	void register_for_events ();
		
	
	/**
	 * This method returns a reference to the event queue.
	 */
	void get_queue ();
		
	
	/**
	 * This method is used to insert an event into the event queue.
	 */
	void insert_event ();
		
	
/**
 * Protected stuff
 */
protected:
	/**
	 * Fields
	 */
	/**
	 * 
	 */
	/**
	 * Constructors
	 */
	/**
	 * Accessor Methods
	 */
	/**
	 * Operations
	 */
/**
 * Private stuff
 */
private:
	/**
	 * Fields
	 */
	/**
	 * Used to determine if the class should be created.
	 */
	static Application *_instance;
	/**
	 * This class holds all the events collected by the input plugins.
	 */
	 CircularQueue* event_queue;
	/**
	 * 
	 */
	/**
	 * Constructors
	 */
	/**
	 * Empty Constructor
	 */
	Application ( ) { };
	/**
	 * Empty Constructor
	 */
	Application(const Application&);
	/**
	 * Empty Constructor
	 */
	Application& operator=(const Application&);

	/**
	 * Empty Constructor
	 */
	virtual ~Application() { _instance = 0; }

	
	/**
	 * Accessor Methods
	 */
	/**
	 * Get the value of event_queue
	 * This class holds all the events collected by the input plugins.
	 * @return the value of event_queue
	 */
	/* CircularQueue get_event_queue ( ); */
		
	
	/**
	 * Set the value of event_queue
	 * This class holds all the events collected by the input plugins.
	 * @param value the value of event_queue
	 */
	/* void set_event_queue (CircularQueue<Event> value ); */
		
	
	/**
	 * Operations
	 */
};
#endif //APPLICATION_H

